/* IMPORT */
import ShoSho from 'shosho';
import { castArray, shortcut2id } from './utils.js';
/* MAIN */
class Shortcuts {
    /* CONSTRUCTOR */
    constructor(options) {
        /* VARIABLES */
        this.registrations = [];
        /* PUBLIC API */
        this.get = () => {
            return this.registrations.map(registration => registration.descriptor);
        };
        this.add = (descriptor) => {
            const descriptors = castArray(descriptor);
            for (const descriptor of descriptors) {
                if (descriptor.shortcut.startsWith('-')) {
                    this.remove(descriptor);
                }
                else if (descriptor.handler) {
                    const dispose = this.shosho.register(descriptor.shortcut, descriptor.handler);
                    const shortcut = shortcut2id(descriptor.shortcut);
                    const registration = { descriptor, dispose, shortcut };
                    this.registrations.push(registration);
                }
                else {
                    console.error(`Cannot register shortcut "${descriptor.shortcut}", which has no handler`);
                }
            }
        };
        this.register = (descriptor) => {
            this.add(descriptor);
            return () => {
                this.remove(descriptor);
            };
        };
        this.remove = (descriptor) => {
            const descriptors = castArray(descriptor);
            for (const descriptor of descriptors) {
                const handler = descriptor.handler;
                const shortcut = shortcut2id(descriptor.shortcut.replace(/^-/, ''));
                this.registrations = this.registrations.filter(registration => {
                    if ((registration.shortcut !== shortcut) || (handler && registration.descriptor.handler !== handler)) {
                        return true;
                    }
                    else {
                        registration.dispose();
                        return false;
                    }
                });
            }
        };
        this.reset = () => {
            this.registrations = [];
            this.shosho.reset();
        };
        this.trigger = (shortcut) => {
            return this.shosho.trigger(shortcut);
        };
        this.start = () => {
            this.shosho.start();
        };
        this.stop = () => {
            this.shosho.stop();
        };
        this.registrations = [];
        this.shosho = new ShoSho(options);
    }
}
/* EXPORT */
export default Shortcuts;
